'' =================================================================================================
''
''   File....... anemometer.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 07 MAY 2010
''
'' =================================================================================================


CON


OBJ


VAR

  long  Cog
  

PUB init(pin, pntr) | ok

'' Start anemometer object
'' -- pin.... active-high input from anemometer
'' -- pntr... address of target variable (updated every 2.5s)

  finalize

  ' set cog parameters
  '
  mask   := |< pin
  dbwait := (clkfreq / 50) >> 1                                 ' 1/2 of max input rate (50Hz)
  period := (clkfreq << 1) + (clkfreq >> 1)                     ' ticks in 2.5 seconds
  vpntr  := pntr                                                ' address of user variable

  ok := Cog := cognew(@anemometer, 0) + 1                       ' start the cog

  return ok


PUB finalize

'' Stops cog (if running)

  if Cog
    cogstop(Cog~ - 1)
    long[vpntr]~


DAT

                        org     0

anemometer              mov     dira, #0
                        mov     t0, cnt                         ' start 2.5s window

anestart                mov     pulses, #0                      ' clear pulse count

scan                    test    mask, ina               wc      ' check input
        if_nc           jmp     #checkwindow                    ' skip if no input
        
incpulses               add     pulses, #1                      ' increment pulse count
debounce                mov     tmp1, dbwait                    ' insert short delay
                        add     tmp1, cnt
                        waitcnt tmp1, #0

checkwindow             mov     tmp1, cnt                       ' grab system cnt
                        subs    tmp1, t0                        ' calculate elapsed ticks
                        abs     tmp1, tmp1
                        cmp     tmp1, period            wc, wz  ' reached period?
        if_b            jmp     #scan                           ' no, scan again

                        adds    t0, period                      ' reset start of 2.5ms window

report                  wrlong  pulses, vpntr                   ' write pulses to hub
                        jmp     #anestart

' --------------------------------------------------------------------------------------------------

mask                    long    0-0                             ' input pin mask   
dbwait                  long    0-0                             ' ticks delay for debouncing
period                  long    0-0                             ' ticks in 2.5s sample window
vpntr                   long    0-0                             ' pointer to user variable

pulses                  res     1                               ' accumulated pulses
t0                      res     1                               ' start of sample window

tmp1                    res     1
tmp2                    res     1

                        fit     492                                    

                        
DAT

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  